;	* * * * * * * * *
;	*  easy_cw.asm  *
;	* * * * * * * * *
;
;	Simple test program for LF beacon transmitter.
;
;	BPSK mode is not implemented. Serial interface
;	is not implemented either. A possible use for the
;	64 byte EEPROM in the CPU could be to store messages
;	that are downloaded via the serial port. Another
;	idea is to feed data to be transmitted into the
;	serial port in real time, using the handshake line
;	for flow control. Help yourself.
;
;	Used assembler: MPASM 1.50
;	Crystal: 4.0 MHz
;
;	Three of the jumpers are used to select one of
;	eight possible messages. Another jumper selects
;	between two different lengths of the long dash
;	(if it is present in the selected message).
;
;	Freeware - use at your own risk.
;
;	NOTE! You MUST change the CW messages in this
;	file and re-assemble if you are going to use
;	the beacon on the air! You have been warned ;-)
;
;	Johan Bodin SM6LKM


;-------------------------------------------------------------------------------
;	Assembler directives
;-------------------------------------------------------------------------------

	title		"LF Beacon"
	subtitle	"Easy_CW"
	processor	16C84
	include		"p16c84.inc"	; EQUs for processor registers etc.
	radix		dec		; decimal radix as default
	errorlevel	0		; show all warnings and errors
	__idlocs	0x2BAD		; hihi...
	__config	_CP_OFF & _PWRTE_OFF & _WDT_ON & _XT_OSC


;-------------------------------------------------------------------------------
;	CW speed
;-------------------------------------------------------------------------------

T_UNIT	equ	2	; number of TMR0 overflow ticks per CW time unit

;	See also prescaler bits in the OPTION register!

;-------------------------------------------------------------------------------
;	Naming of RAM variables
;-------------------------------------------------------------------------------

	CBLOCK	0x0C	; we have 36 bytes of RAM in the area 0x0C - 0x2F
	  delcnt	; dot/dash/space delay tick counter
	  temp
	ENDC


;-------------------------------------------------------------------------------
;	I/O pin mapping and pin read/write macros
;-------------------------------------------------------------------------------

;	PORTA pins

;	RA4	Indicator LED, output, active low
;	RA3	Serial tx, output, idling low -> -12V on RS232 line
;	RA2	Serial CTS, output, 0 = stop, 1 = continue
;	RA1	BPSK key, output
;	RA0	CW key, output, normally active high (see CW invert jumper)


;	PORTA pin read/write macros

#define	p_led	PORTA,4
#define	p_txd	PORTA,3
#define	p_cts	PORTA,2
#define	p_bpsk	PORTA,1
#define	p_cw	PORTA,0


;	PORTB pins

;	RB7	Mode configuration jumper #7 or future I/O
;	RB6	Mode configuration jumper #6 or future I/O
;	RB5	Mode configuration jumper #5 or future I/O
;	RB4	Mode configuration jumper #4 or future I/O
;	RB3	Mode configuration jumper #3 or future I/O
;	RB2	Mode configuration jumper #2 or future I/O
;	RB1	Mode configuration jumper #1 or future I/O
;	RB0	Serial rx, input, idling high (transitor Q1 inverts)


;	PORTB pin read/write macros

#define	p_jp7	PORTB,7
#define	p_jp6	PORTB,6
#define	p_jp5	PORTB,5
#define	p_jp4	PORTB,4
#define	p_jp3	PORTB,3
#define	p_jp2	PORTB,2
#define	p_jp1	PORTB,1
#define	p_rxd	PORTB,0


;	Pin directions after power-up, 0=output, 1=input

PUTRISA	equ	B'00000000'	; all PORTA pins are outputs
PUTRISB	equ	B'11111111'	; all PORTB pins are inputs


;	Output pin levels after power-up

PUPORTA	equ	B'00010000'	; LED off, CW key off, TXD idle, CTS off
PUPORTB	equ	B'00000000'	; unimportant since all PORTB pins are inputs


;-------------------------------------------------------------------------------
;	CW character macros
;-------------------------------------------------------------------------------

;	dot   = 1T key down + 1T key up
;	dot_  = 1T key down + 3T key up (for inter-character space)
;	dash  = 3T key down + 1T key up
;	dash_ = 3T key down + 3T key up (for inter-character space)

__A	macro
	call	dot
	call	dash_
	endm

__B	macro
	call	dash
	call	dot
	call	dot
	call	dot_
	endm

__C	macro
	call	dash
	call	dot
	call	dash
	call	dot_
	endm

__D	macro
	call	dash
	call	dot
	call	dot_
	endm

__E	macro
	call	dot_
	endm

__F	macro
	call	dot
	call	dot
	call	dash
	call	dot_
	endm

__G	macro
	call	dash
	call	dash
	call	dot_
	endm

__H	macro
	call	dot
	call	dot
	call	dot
	call	dot_
	endm

__I	macro
	call	dot
	call	dot_
	endm

__J	macro
	call	dot
	call	dash
	call	dash
	call	dash_
	endm

__K	macro
	call	dash
	call	dot
	call	dash_
	endm

__L	macro
	call	dot
	call	dash
	call	dot
	call	dot_
	endm

__M	macro
	call	dash
	call	dash_
	endm

__N	macro
	call	dash
	call	dot_
	endm

__O	macro
	call	dash
	call	dash
	call	dash_
	endm

__P	macro
	call	dot
	call	dash
	call	dash
	call	dot_
	endm

__Q	macro
	call	dash
	call	dash
	call	dot
	call	dash_
	endm

__R	macro
	call	dot
	call	dash
	call	dot_
	endm

__S	macro
	call	dot
	call	dot
	call	dot_
	endm

__T	macro
	call	dash_
	endm

__U	macro
	call	dot
	call	dot
	call	dash_
	endm

__V	macro
	call	dot
	call	dot
	call	dot
	call	dash_
	endm

__W	macro
	call	dot
	call	dash
	call	dash_
	endm

__X	macro
	call	dash
	call	dot
	call	dot
	call	dash_
	endm

__Y	macro
	call	dash
	call	dot
	call	dash
	call	dash_
	endm

__Z	macro
	call	dash
	call	dash
	call	dot
	call	dot_
	endm

__AA	macro
	call	dot
	call	dash
	call	dash
	call	dot
	call	dash_
	endm

__AE	macro
	call	dot
	call	dash
	call	dot
	call	dash_
	endm

__OE	macro
	call	dash
	call	dash
	call	dash
	call	dot_
	endm

__0	macro
	call	dash
	call	dash
	call	dash
	call	dash
	call	dash_
	endm

__1	macro
	call	dot
	call	dash
	call	dash
	call	dash
	call	dash_
	endm

__2	macro
	call	dot
	call	dot
	call	dash
	call	dash
	call	dash_
	endm

__3	macro
	call	dot
	call	dot
	call	dot
	call	dash
	call	dash_
	endm

__4	macro
	call	dot
	call	dot
	call	dot
	call	dot
	call	dash_
	endm

__5	macro
	call	dot
	call	dot
	call	dot
	call	dot
	call	dot_
	endm

__6	macro
	call	dash
	call	dot
	call	dot
	call	dot
	call	dot_
	endm

__7	macro
	call	dash
	call	dash
	call	dot
	call	dot
	call	dot_
	endm

__8	macro
	call	dash
	call	dash
	call	dash
	call	dot
	call	dot_
	endm

__9	macro
	call	dash
	call	dash
	call	dash
	call	dash
	call	dot_
	endm

__error	macro
	call	dot
	call	dot
	call	dot
	call	dot
	call	dot
	call	dot
	call	dot
	call	dot_
	endm

__atten	macro
	call	dash
	call	dot
	call	dash
	call	dot
	call	dash_
	endm

__excla	macro
	call	dot
	call	dot
	call	dash
	call	dash
	call	dot_
	endm

__plus	macro
	call	dot
	call	dash
	call	dot
	call	dash
	call	dot_
	endm

__comma	macro
	call	dash
	call	dash
	call	dot
	call	dot
	call	dash
	call	dash_
	endm

__minus	macro
	call	dash
	call	dot
	call	dot
	call	dot
	call	dot
	call	dash_
	endm

__perio	macro
	call	dot
	call	dash
	call	dot
	call	dash
	call	dot
	call	dash_
	endm

__slash	macro
	call	dash
	call	dot
	call	dot
	call	dash
	call	dot_
	endm

__equal	macro
	call	dash
	call	dot
	call	dot
	call	dot
	call	dash_
	endm

__quest	macro
	call	dot
	call	dot
	call	dash
	call	dash
	call	dot
	call	dot_
	endm

__@	macro
	call	dot
	call	dot
	call	dot
	call	dash
	call	dot
	call	dash_
	endm

__wait	macro
	call	dot
	call	dash
	call	dot
	call	dot
	call	dot_
	endm


;-------------------------------------------------------------------------------
;	Reset entry point
;-------------------------------------------------------------------------------

	ORG	0x0000

;	Set pin output latches

	movlw	PUPORTA
	movwf	PORTA
	movlw	PUPORTB
	movwf	PORTB


;	Set pin directions

	call	setpindir

;	Initialize the OPTION register
;
;	  /RBPU = 0      -> PORTB internal pull-up on
;	  INTEDG = 0     -> RB0/INT pin interrupt on falling edge
;	  T0CS = 0       -> TMR0 internal clock (not RA4/T0CKI pin)
;	  T0SE = 0       -> TMR0 source edge, unimportant when T0CS = 0
;	  PSA = 0        -> prescaler connected to TMR0 (instead of WDT)
;	  PS2..PS0 = 111 -> prescaler divides by 256 -> 65.536 ms tick

	clrwdt			; according to "the book" a clrwdt has to...
	bsf	_RP0		; ...be executed before the prescaler is...
	movlw	B'00000111'	; ...moved from WDT to TMR0 in order to...
	movwf	0x7F & OPTI	; ...avoid an unintentional WDT reset EVEN...
	bcf	_RP0		; ...if the WDT is disabled!

	goto	mainlooptop


;-------------------------------------------------------------------------------
;	Main program loop
;-------------------------------------------------------------------------------

mainlooptop:
	movf	PORTB,w		; read PORTB to get mode jumpers
	movwf	temp		; copy to RAM
	rrf	temp,f		; shift 5 bits to the right
	rrf	temp,f
	rrf	temp,f
	rrf	temp,f
	rrf	temp,w		; place result of final rrf into W
	andlw	0x07		; the 3 rightmost jumpers at the board edge
	clrf	PCLATH		; jump table is below 0x100
	addwf	PCL,f		; add to low byte of program counter

	goto	msg0
	goto	msg1
	goto	msg2
	goto	msg3
	goto	msg4
	goto	msg5
	goto	msg6
	goto	msg7


;*******************************************************************************
;	"VVV DE SM6LKM QTH JO67MR"
;*******************************************************************************

msg0:
	call	delay_3t		; 4 from bottom + this one = 7T

	__V
	__V
	__V

	call	delay_4t

	__D
	__E

	call	delay_4t

	__S
	__M
	__6
	__L
	__K
	__M

	call	delay_4t

	__Q
	__T
	__H

	call	delay_4t

	__J
	__O
	__6
	__7
	__M
	__R

	call	delay_1t		; now we have 4T space

	goto	mainlooptop


;*******************************************************************************
;	"DE SM6LKM QTH JO67MR (long dash)"
;*******************************************************************************

msg1:
	call	delay_3t		; 4 from bottom + this one = 7T

	__D
	__E

	call	delay_4t

	__S
	__M
	__6
	__L
	__K
	__M

	call	delay_4t

	__Q
	__T
	__H

	call	delay_4t

	__J
	__O
	__6
	__7
	__M
	__R

	goto	send_the_long_dash


;*******************************************************************************
;	"SM6LKM BEACON 136.65 KHZ JO67MR (long dash)"
;*******************************************************************************

msg2:
	call	delay_3t		; 4 from bottom + this one = 7T

	__S
	__M
	__6
	__L
	__K
	__M				; this has a 3T space at its end

	call	delay_4t

	__B
	__E
	__A
	__C
	__O
	__N

	call	delay_4t

	__1
	__3
	__6
	__perio
	__6
	__5
	__K
	__H
	__Z

	call	delay_4t

	__J
	__O
	__6
	__7
	__M
	__R

	goto	send_the_long_dash


;*******************************************************************************
;	"TRL" (19T IWS)
;*******************************************************************************

msg3:
	call	delay_3t		; 16 from bottom + this one = 19T

	__T
	__R
	__L				; 3T space at the end of "L"

	call	delay_4t		; +4 = 7T
	call	delay_4t		; +4 = 11T
	call	delay_4t		; +4 = 15T
	call	delay_1t		; +1 = 16T

	goto	mainlooptop


;*******************************************************************************
;	"JHB" (19T IWS)
;*******************************************************************************

msg4:
	call	delay_3t		; 16 from bottom + this one = 19T

	__J
	__H
	__B				; this has a 3T space at its end

	call	delay_4t		; +4 = 7T
	call	delay_4t		; +4 = 11T
	call	delay_4t		; +4 = 15T
	call	delay_1t		; +1 = 16T

	goto	mainlooptop


;*******************************************************************************
;	"HEJ DE SM6LKM (long dash)"
;*******************************************************************************

msg5:
	call	delay_3t		; 4 from bottom + this one = 7T

	__H
	__E
	__J

	call	delay_4t

	__D
	__E

	call	delay_4t

	__S
	__M
	__6
	__L
	__K
	__M				; this has a 3T space at its end

	goto	send_the_long_dash


;*******************************************************************************
;	"SM6LKM"
;*******************************************************************************

msg6:
	call	delay_3t		; 4 from bottom + this one = 7T

	__S
	__M
	__6
	__L
	__K
	__M				; this has a 3T space at its end

	call	delay_1t		; now we have 4T space

	goto	mainlooptop


;*******************************************************************************
;	"LKM"
;*******************************************************************************

msg7:
	call	delay_3t		; 4 from bottom + this one = 7T

	__L
	__K
	__M				; this has a 3T space at its end

	call	delay_1t		; now we have 4T space

	goto	mainlooptop

;------


send_the_long_dash:

	call	delay_4t

	bsf	p_cw		; key down
	movlw	76		; 76*65.536 ms = 4.98 sec.
	btfss	p_jp4		; leftmost jumper near board edge on?
	movlw	152		; yes, use 9.96 sec. instead
	call	dodelay		; send long RDF dash
	bcf	p_cw		; key up

	call	delay_4t

	goto	mainlooptop

;-------------------------------------------------------------------------------
;	Dot/dash subroutines
;-------------------------------------------------------------------------------

dot:				; normal dot
	bsf	p_cw		; key down
	call	delay_1t	; wait one time unit
	bcf	p_cw		; key up
	call	delay_1t	; wait one time unit
	return
dot_:				; dot at end of character
	bsf	p_cw
	call	delay_1t
	bcf	p_cw
	call	delay_3t
	return

dash:				; normal dash
	bsf	p_cw
	call	delay_3t
	bcf	p_cw
	call	delay_1t
	return

dash_:				; dash at end of character
	bsf	p_cw
	call	delay_3t
	bcf	p_cw
	call	delay_3t
	return


;-------------------------------------------------------------------------------
;	CW dot/dash/space delay routine
;-------------------------------------------------------------------------------

delay_4t:
	movlw	4*T_UNIT	; used for inter-word space (7T)
	goto	dodelay

delay_3t:
	movlw	3*T_UNIT
	goto	dodelay

delay_1t:
	movlw	1*T_UNIT

dodelay:
	movwf	delcnt		; number of TMR0 ticks to wait

	btfss	p_cw		; copy key to LED
	bsf	p_led
	btfsc	p_cw
	bcf	p_led

	call	setpindir	; keep pins pointing in the right directions

wait4tmr:
	clrwdt			; feed watchdog timer
	btfss	_T0IF		; has TMR0 wrapped around
	goto	wait4tmr	; no, wait for 0xFF to 0x00 roll-over

	bcf	_T0IF		; clear TMR0 flag
	decfsz	delcnt,f	; one tick eaten, more to do?
	goto	wait4tmr	; yes

	return			; no, back to business


;-------------------------------------------------------------------------------
;	Subroutine that uppdates pin directions
;-------------------------------------------------------------------------------

setpindir:
	bsf	_RP0		; select RAM page 1
	movlw	PUTRISA		; pin directions for PORTA
	movwf	0x7F & TRISA
	movlw	PUTRISB		; pin directions for PORTB
	movwf	0x7F & TRISB
	bcf	_RP0		; RAM page 0 should be mapped in as default
	return


;-------------------------------------------------------------------------------
	END
